{% load django_tables2 %}
{% if enable_merge and user.is_authenticated %}
<form action="{% url 'browsing:merge_objects' %}" method="POST">
    {% csrf_token %}
    <input type="hidden" name="model_name" value="{{ entity }}"/>
    <input type="hidden" name="app_name" value="{{ app_name }}"/>
{% endif %}
    <table class="table table-hover" {% if table.attrs %} {{ table.attrs.as_html }}{% endif %}>
        {% block table.thead %}
        {% if table.show_header %}
        <thead>
            <tr>
            {% for column in table.columns %}
                {% if column.orderable %}
                <th {{ column.attrs.th.as_html }}><a href="{% querystring table.prefixed_order_by_field=column.order_by_alias.next %}">{{ column.header }}</a></th>
                {% else %}
                <th {{ column.attrs.th.as_html }}>{{ column.header }}</th>
                {% endif %}
            {% endfor %}
            </tr>
        </thead>
        {% endif %}
        {% endblock table.thead %}
        {% block table.tbody %}
        <tbody>
            {% for row in table.page.object_list|default:table.rows %} {# support pagination #}
            {% block table.tbody.row %}
            <tr {{ row.attrs.as_html }}>
                {% for column, cell in row.items %}
                <td {{ column.attrs.td.as_html }}>{% if column.localize == None %}{{ cell }}{% else %}{% if column.localize %}{{ cell|localize }}{% else %}{{ cell|unlocalize }}{% endif %}{% endif %}</td>

                {% endfor %}
            </tr>
            {% endblock table.tbody.row %}
            {% empty %}
            {% if table.empty_text %}
            {% block table.tbody.empty_text %}
            <tr><td colspan="{{ table.columns|length }}">{{ table.empty_text }}</td></tr>
            {% endblock table.tbody.empty_text %}
            {% endif %}
            {% endfor %}
        </tbody>
        {% endblock table.tbody %}
        {% block table.tfoot %}
        {% if table.has_footer %}
        <tfoot>
            <tr>
            {% for column in table.columns %}
                <td>{{ column.footer }}</td>
            {% endfor %}
            </tr>
        </tfoot>
        {% endif %}
        {% endblock table.tfoot %}
    </table>
{% if enable_merge and user.is_authenticated %}
    <input type="submit" value="merge" />
    {% endif %}
</form>