{% load django_tables2 %}
<div class="dropdown">
    <button class="btn btn-default dropdown-toggle dropdown-custom" type="button" id="dropdownMenu1" data-toggle="dropdown" aria-haspopup="true" aria-expanded="true">
        <i class="fas fa-download"></i>
        Download
        <span class="caret"></span>
    </button>
    <ul class="dropdown-menu" aria-labelledby="dropdownMenu1">
        <li><a class="dropdown-item" href="{% querystring %}&_export=csv">CSV</a></li>
        <li><a class="dropdown-item" href="{% querystring %}&_export=json">JSON</a></li>
    </ul>
</div>